<?php
class Setup {
	private $lSettingsFields = array(	'dreamboxName' 			=> array('type' => 'string','required' => true),
										'dreamboxIP' 			=> array('type' => 'string','required' => true),
										'dreamboxWebPort'		=> array('type' => 'int','required' => true),
										'dreamboxUserName'	 	=> array('type' => 'string','required' => false),
										'dreamboxPassword' 		=> array('type' => 'string','required' => false),
										'dreamboxEnigmaVersion' => array('type' => 'string','required' => true),
										'dreamboxDualTuner' 	=> array('type' => 'int','required' => true),
										'privateModusUsername' 	=> array('type' => 'string','required' => false),
										'privateModusPassword'	=> array('type' => 'string','required' => false),
										'vlcLocation' 			=> array('type' => 'string','required' => true),
										'vlcIP' 				=> array('type' => 'string','required' => true),
										'vlcLanStreamPort' 		=> array('type' => 'int','required' => true),
										'moviesPath' 			=> array('type' => 'string','required' => false),
										'vlcAudioLanguage' 		=> array('type' => 'string','required' => false),
										'vlcSubtitleLanguage' 	=> array('type' => 'string','required' => false),
										'mediaInfoLocation' 	=> array('type' => 'string','required' => false),
									);
	private $lCurrentSettings;
	private $needsUpdate;

	function __construct() {
		$this->lCurrentSettings = array();
		$lConfigFiles = array('Settings.class.sample.php','Settings.class.php');
		
		foreach($lConfigFiles as $lConfigFile) {
			if (file_exists($lConfigFile)) {
				$lFileContents = file($lConfigFile);
				foreach ($lFileContents as $lLine) {
					if (substr($lLine = trim($lLine),0,14) == 'private static') {
						$lData = explode('=',substr($lLine,14));
						$lVariable = substr(trim($lData[0]),1);
						$lKey = trim($lData[1]);
						if (substr($lKey,-1) == ';') $lKey = trim(substr($lKey,0,-1));
						if (substr($lKey,0,1) == '"') {
							$lKey = substr($lKey,1,-1);
						}
						$this->lCurrentSettings[$lVariable] = $lKey;
					}
				}
			}
		}
		$this->needsUpdate = !$this->existsSettingsFile() || ($this->getSettingsVersion() < $this->getSetupVersion());
	}

	private function existsSettingsFile() {
		return file_exists('Settings.class.php');
	}
	
	private function getSetupVersion() {
		return trim(VERSION);
	}
	
	private function getSettingsVersion() {
		return $this->lCurrentSettings['version'];
	}

	private function validateSettings($pFormData) {
		$lReturnArray = array();
		$lErrorMessages = $this->checkWriteSettings();
		$lReturnArray = array_merge($lReturnArray,$lErrorMessages);
		
		foreach ($pFormData as $lField => $lValue) {
			if (!empty($this->lSettingsFields[$lField])) {
		
				if ($this->lSettingsFields[$lField]['required'] && $lValue == '') {
					$lReturnArray[$lField] = 'Field is required';
				} elseif ($this->lSettingsFields[$lField]['type'] == 'int' && !is_numeric($lValue)) {
					$lReturnArray[$lField] = 'Invalid input';
				} elseif ($this->lSettingsFields[$lField]['type'] == 'string' && !is_string($lValue)) {
					$lReturnArray[$lField] = 'Invalid input';					
				}
			}
		}
		return $lReturnArray;
	}
	
	private function findIPNumbers() {
		$lReturnArray = array();
		$lCMD = shell_exec('which ifconfig');
		$lData = shell_exec($lCMD);
		$lData = explode("\n",$lData);
		foreach ($lData as $lLine) {
			$lLine = trim($lLine);
			if (stripos($lLine,'Link encap:') !== false) {
				$lDevice = trim(substr($lLine,0,stripos($lLine,' ')));
			}			
			if (stripos($lLine,'inet addr:') !== false) {
				$lIP = trim(substr($lLine,10,stripos($lLine,' ',10)-10));
				$lReturnArray[$lDevice] = $lIP;
			}
		}
		return $lReturnArray;
	}
	
	private function findVLC() {
		if (( $lVLC = trim(shell_exec('which cvlc'))) != '') return $lVLC;
		if (( $lVLC = trim(shell_exec('which vlc'))) != '') return $lVLC;
		return '';
	}
	
	private function findMediaInfo() {
		return trim(shell_exec('which mediainfo'));
	}
	
	public function configOutDated() {
		return $this->needsUpdate;
	}

	public function getVariable($pName) {
		$lReturnValue = '';
		if (($pName = trim($pName)) != '') {
			if (!empty($this->lSettingsFields[$pName])) {
				$lFieldData = $this->lSettingsFields[$pName];
				if (!empty($this->lCurrentSettings[$pName])) {
					$lReturnValue = $this->lCurrentSettings[$pName];
				}
				
				switch ($pName) {
					case 'dreamboxEnigmaVersion':
						if ($lReturnValue == '') {
							$lReturnValue = array('enigma1','enigma2');
						} else {
							$lReturnValue = array($lReturnValue);
						}
					break;
					
					case 'vlcLocation':
						if ($lReturnValue == '') {
							$lReturnValue = $this->findVLC();
						}
					break;
					case 'vlcIP':
						if ($lReturnValue == '') {
							$lReturnValue = $this->findIPNumbers();
						} else {
							$lReturnValue = array($lReturnValue);
						}
					break;
					case 'mediaInfoLocation':
						if ($lReturnValue == '') {
							$lReturnValue = $this->findMediaInfo();
						}
					break;
				}
			}
		}
		return $lReturnValue;
	}
	
	public function checkWriteSettings() {
		$lErrorMessages = array();
		if (!file_exists('Settings.class.php')) {
			if (!@touch('Settings.class.php')) {
				$lErrorMessages[] = 'Create an empty Settings.class.php file and make it writable for the web server';
			} else {
				unlink('Settings.class.php');
			}
		}
		else if (file_exists('Settings.class.php') && !@touch('Settings.class.php')) {
			$lErrorMessages[] = 'Settings file Settings.class.php is not writable!';
			
		}
		else if (!(@touch('iphone/stream/test.txt') && unlink('iphone/stream/test.txt'))) {
			$lErrorMessages[] = 'The folder iphone/stream is not writable for the web server!';
		}
		else if (!(touch('picon/test.txt') && unlink('picon/test.txt'))) {
			$lErrorMessages[] = 'The folder picon is not writable for the web server!';
		}
		return $lErrorMessages;
	}

	public function updateSettings($pFormData) {
		$lAjaxResponse  = new xajaxResponse();
		
		$lReturnValue = $this->validateSettings($pFormData);
		if (count($lReturnValue) > 0) {
			$lAjaxResponse->assign("message","innerHTML","<p>Incorrect data:<br />");
			foreach ($lReturnValue as $lField => $lMessage) {
				if ($lField != '') $lAjaxResponse->script("jQuery('#message').append(jQuery('label[for=\"" . $lField . "\"]').text() + ': ')");
				$lAjaxResponse->script("jQuery('#message').append('$lMessage<br />')");
			}
			$lAjaxResponse->script("jQuery('#message').append('</p>')");
			$lAjaxResponse->script("jQuery('#message').dialog({modal: true,closeOnEscape: false});");			
		} else {
			foreach($this->lCurrentSettings as $lField => $lData) {
				switch ($lField) {
					case 'version':
						$pFormData[$lField] = $this->getSetupVersion();
					break;
					case 'privateModus':
						$pFormData[$lField] = ($pFormData['privateModusUsername'] != '' && $pFormData['privateModusPassword'] != '');
					break;
					default:
						if (!array_key_exists($lField,$pFormData)) $pFormData[$lField] = $lData;
					break;
				}
				
			}

			$lFileContents = file('Settings.class.sample.php');
			for ($i = 0; $i < count($lFileContents); $i++) {
				$lLine = $lFileContents[$i];
				if (substr($lLine = trim($lLine),0,14) == 'private static') {
					$lData = explode('=',substr($lLine,14));
					$lVariable = substr(trim($lData[0]),1);
					
					$lStartPos = stripos($lFileContents[$i],$lVariable)-1;
					$lStopPos = stripos($lFileContents[$i],';',$lStartPos)+1;
					
					$lSearch = substr($lFileContents[$i],$lStartPos,$lStopPos-$lStartPos);
					
					if (empty($pFormData[$lVariable]) || !is_numeric($pFormData[$lVariable])) $pFormData[$lVariable] = '"' . $pFormData[$lVariable] . '"';
					$lReplace = '$' . $lVariable . ' = ' . $pFormData[$lVariable] . ';';
					
					$lFileContents[$i] = str_replace($lSearch, $lReplace, $lFileContents[$i]);
				}
			}
			if (file_put_contents('Settings.class.php',implode('',$lFileContents))) {
				unset($_SESSION["ReStream2.0"]["Dreambox"]);
				$lAjaxResponse->assign("message","innerHTML","<p>Settings are configured. The page will refresh in 10 seconds.</p>");
				$lAjaxResponse->script("setTimeout(function() {location.reload();},10000)");
			} else {
				$lAjaxResponse->assign("message","innerHTML","<p>Error!</br >Could not write the settings file</p>");
			}
		}
		$lAjaxResponse->script("jQuery('#message').dialog({modal: true,closeOnEscape: false});");
		return $lAjaxResponse;
	}
}